@@ -13,61 +13,55 @@
  * open or locked and optionally notifies the user upon state changes.
  *
  *
- * It is a shared librarary that implements a derived HDHomePluginItem (a 
- * desktop widget), with the help of the HD_DEFINE_PLUGIN_MODULE() macro, which 
- * also implements some standard functions that all desktop widget libraries 
- * should export.
+ * Copyright (c) 2010 Hauke Lampe
+ * This code is licensed under a MIT-style License, see file COPYING
  *
- * Note that the API contains the name "home plugin", but the correct name 
- * is now "desktop widget".
- * 
- * As with other custom GTK+ widgets, drawing happens only in a handler for the 
- * expose-event signal. This is triggered by invalidating the desktop widget's
- * area.
- *
- */
+ **/
+
+/**
+  TODO:
+
+   * maybe important, requires deeper code changes:
+   - add config setting: use any connection/WLAN only
+     - we don't use bearer information yet
+   - download another file with DHCP/WLAN client counters
+     - better get them merged into the same file
+     - download/processing function evolved from synchronous
+       call, not suited for multiple URLs
+     - use pango to draw numbers over icon
+
+   * maybe good, requires reading more docs:
+   - check settings dialog for object/memory leaks
+     - which parts are freed automatically, which are not?
+     - is g_unref() the right tool?
+   - check return code from g_timeout_add_seconds/g_signal_connect
+     - right now, we wouldn't know if start_timer()/install_handler() failed
+     - what to do in case of error?
+   - does g_source_remove destroy the source or does it leak?
+     - what data structures does a source use?
+
+   * could be useful, requires more thinking:
+   - aggregate calls to force_redraw()
+     - right now, we call force_redraw() on any possible change
+   - skip draw if not on active desktop
+     - there's an event for that
+     - don't skip update or banner
+   - move constants from .h to configuration settings or autoconf options
+
+   * style issues, don't affect features:
+   - use more HildonCaption in settings dialog?
+     - checkbox buttons look fine without Caption
+     - PickerButton could use a larger font
+
+   * nice to have:
+   - draw nice icons
+     - winni already did some work on new icons
+   - support multiple instances
+     - to support future extensions
+     - not much use now
+
+ **/
 
-// TODO:
-//
-// * maybe important, requires deeper code changes:
-// - add config setting: use any connection/WLAN only
-//   - we don't use bearer information yet
-// - download another file with DHCP/WLAN client counters
-//   - better get them merged into the same file
-//   - download/processing function evolved from synchronous
-//     call, not suited for multiple URLs
-//   - use pango to draw numbers over icon
-//
-// * maybe good, requires reading more docs:
-// - sprinkle with type-checking macros and g_assert
-//   - how do they work anyway?
-// - check settings dialog for object/memory leaks
-//   - which parts are freed automatically, which are not?
-//   - is g_unref() the right tool?
-// - check return code from g_timeout_add_seconds/g_signal_connect
-//   - right now, we wouldn't know if start_timer()/install_handler() failed
-//   - what to do in case of error?
-// - does g_source_remove destroy the source or does it leak?
-//   - what data structures does a source use?
-//
-// * could be useful, requires more thinking:
-// - aggregate calls to force_redraw()
-//   - right now, we call force_redraw() on any possible change
-// - skip draw if not on active desktop
-//   - there's an event for that
-//   - don't skip update or banner
-//
-// * style issues, don't affect features:
-// - use more HildonCaption in settings dialog?
-//   - checkbox buttons look fine without Caption
-//   - PickerButton could use a larger font
-//
-// * nice to have:
-// - draw nice icons
-//   - winni already did some work on new icons
-// - support multiple instances
-//   - to support future extensions
-//   - not much use now
 
 
 
@@ -134,6 +128,8 @@ HD_DEFINE_PLUGIN_MODULE (CCCHHDoorIsDesktopWidget, ccchh_dooris_desktop_widget,
 static inline gboolean
 start_update_timer(CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	if (!self->auto_update)
 		return FALSE;
 
@@ -159,6 +155,8 @@ start_update_timer(CCCHHDoorIsDesktopWidget *self)
 static inline gboolean
 stop_update_timer(CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	// stop timer if it was running
 	if (self->timeout_handler) {
 		g_debug(WIDGET_NAME": stopping update timer");
@@ -177,6 +175,8 @@ stop_update_timer(CCCHHDoorIsDesktopWidget *self)
 static inline gboolean
 install_connection_handler(CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	if (!self->connection_handler) {
 		// set connected to FALSE, initial event may set it to TRUE
 		self->connected = FALSE;
@@ -199,6 +199,8 @@ install_connection_handler(CCCHHDoorIsDesktopWidget *self)
 static inline gboolean
 remove_connection_handler(CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	if (self->connection_handler) {
 		g_object_set (self->connection, "automatic-connection-events", FALSE, NULL);
 		g_signal_handler_disconnect (self->connection, self->connection_handler);
@@ -218,6 +220,8 @@ remove_connection_handler(CCCHHDoorIsDesktopWidget *self)
 static void
 load_settings(CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	GError *error;
 
 	error = NULL;
@@ -328,6 +332,8 @@ load_settings(CCCHHDoorIsDesktopWidget *self)
 static void
 save_settings(CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	GError *error;
 
 	error = NULL;
@@ -394,6 +400,8 @@ save_settings(CCCHHDoorIsDesktopWidget *self)
 static void
 draw (CCCHHDoorIsDesktopWidget *self, cairo_t *cr)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	g_debug(WIDGET_NAME": draw()");
 
 	if (!self->image_current) {
@@ -441,6 +449,8 @@ if (!self->timeout_handler) {
 static void
 show_status_banner (CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	time_t timenow, timediff;
 	gchar *text, *ts_text;
 
@@ -498,6 +508,8 @@ show_status_banner (CCCHHDoorIsDesktopWidget *self)
 static void
 force_redraw (CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	g_debug(WIDGET_NAME": force_redraw()");
 
 	/* Invalidate the drawing: */
@@ -527,6 +539,8 @@ force_redraw (CCCHHDoorIsDesktopWidget *self)
 static inline gboolean
 fetch_update_lock (CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	if (!g_static_mutex_trylock(&(self->fetch_lock))) {
 		return FALSE;
 	}
@@ -543,6 +557,8 @@ fetch_update_lock (CCCHHDoorIsDesktopWidget *self)
 static inline void
 fetch_update_unlock (CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	self->current_alpha = IMAGE_ALPHA_NORMAL;
 	force_redraw(self);
 
@@ -554,9 +570,14 @@ fetch_update_unlock (CCCHHDoorIsDesktopWidget *self)
 static void
 fetch_update_start (CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	g_debug(WIDGET_NAME": fetch_update_start()");
 
-	if (!self->connected) {
+	// TODO: fix conn_wlan_only logic here
+	//       for now, update anyway if force_banner is set
+	//       (indicating a button press)
+	if (!self->connected && !self->force_banner) {
 		g_debug(WIDGET_NAME": not connected, not fetching new state");
 		return;
 	}
@@ -749,6 +770,8 @@ static void
 fetch_update_process (CCCHHDoorIsDesktopWidget *self, dooris_state_t new_state,
 	gulong new_change_ts, gulong new_report_ts)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	time_t timenow;
 
 	g_debug(WIDGET_NAME": fetch_update_process()");
@@ -945,8 +968,10 @@ ccchh_dooris_desktop_widget_on_button_release_event (GtkWidget *widget,
 	// request connection from libconic, if offline and
 	// force_connection is enabled
 	// TODO: this does not work immediately, the call to update()
-	// will still see !connected. maybe wait a bit or use a timer event?
-	if (!self->connected && self->force_connection) {
+	//       will still see !connected. maybe wait a bit or use a timer event?
+	// TODO: conn_wlan_only: force libconic to use only WLAN IAPs.
+	//       for now, do nothing if conn_wlan_only is set
+	if (!self->conn_wlan_only && !self->connected && self->force_connection) {
 		g_debug(WIDGET_NAME": not connected, requesting connection");
 		/* Request connection and check for the result */
 		gboolean success = con_ic_connection_connect(self->connection,
@@ -1007,6 +1032,8 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 			if (iap_id)
 				self->disconnecting_iap = g_strdup(iap_id);
 
+			// TODO: "disconnecting_iap" logic loses previous connection
+			// state, forces unnecessary redraws and timer resets when roaming
 			self->connected = FALSE;
 
 			break;
@@ -1015,7 +1042,13 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 
 			// we are online now
 
-			if (!self->connected) {
+			// ignore online event if bearer type string does not begin
+			// with "WLAN_" and conn_wlan_only option is set
+			if (self->conn_wlan_only &&
+					strncmp(con_ic_event_get_bearer_type(CON_IC_EVENT(event)), "WLAN_", 5)) {
+				g_debug(WIDGET_NAME": ignoring online event from non-WLAN IAP: conn_wlan_only set");
+
+			} else if (!self->connected) {
 				self->connected = TRUE;
 				g_debug(WIDGET_NAME": new connection state: online");
 
@@ -1033,7 +1066,6 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 			if (iap_id && self->disconnecting_iap && self->connected &&
 					!strcmp(iap_id, self->disconnecting_iap))
 			{
-
 				g_debug(WIDGET_NAME": ignoring offline event from disconnecting IAP");
 
 			} else {
@@ -1071,8 +1103,8 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
   gint response_id, gpointer user_data)
 {
 	CCCHHDoorIsDesktopWidget *self = CCCHH_DOORIS_DESKTOP_WIDGET (user_data);
-	g_assert (self->settings_window);
-	g_assert (self->settings_window == GTK_WIDGET (dialog));
+	g_assert(self->settings_window);
+	g_assert(self->settings_window == GTK_WIDGET (dialog));
   
 	g_debug(WIDGET_NAME": ccchh_dooris_desktop_widget_on_settings_dialog_response()");
 
@@ -1150,6 +1182,7 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 
 				// force connection state to online
 				// redraw if necessary to remove disconnection overlay
+				// TODO: (maybe) fix conn_wlan_only logic here
 				if (!self->connected) {
 					self->connected = TRUE;
 					force_redraw(self);
@@ -1496,6 +1529,8 @@ ccchh_dooris_desktop_widget_class_finalize (CCCHHDoorIsDesktopWidgetClass *klass
 static void
 ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 {
+	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
+
 	g_debug(WIDGET_NAME": ccchh_dooris_desktop_widget_init()");
 
 /*
@@ -1514,6 +1549,7 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 	self->use_connection = TRUE;
 	self->force_connection = TRUE;
 	self->auto_update = TRUE;
+	self->conn_wlan_only = FALSE;
 
 	// initialize state
 	// this combination ensures that update() sets an image on first call
