@@ -20,9 +20,9 @@
 #define REPORT_TIMEOUT 7200
 
 // async I/O timeouts in seconds
-#define TIMEOUT_CONNECT 5
-#define TIMEOUT_READ	30
-#define TIMEOUT_CLOSE	5
+#define TIMEOUT_CONNECT 15
+#define TIMEOUT_READ	60
+#define TIMEOUT_CLOSE	15
 
 // max download size in bytes
 #define MAX_HTTP_SIZE 1024
@@ -53,12 +53,16 @@
 #define IMAGE_ALPHA_PROGRESS	0.65
 
 
-G_BEGIN_DECLS
-
-
+#include <glib.h>
 #include <libhildondesktop/libhildondesktop.h>
 #include <conic.h> /* libconic - Internet Connectivity library */
 #include <gconf/gconf-client.h>
+#include <libgnomevfs/gnome-vfs.h>
+
+
+
+G_BEGIN_DECLS
+
 
 #define TYPE_CCCHH_DOORIS_DESKTOP_WIDGET ccchh_dooris_desktop_widget_get_type()
 
@@ -117,6 +121,7 @@ typedef struct {
 	gboolean use_connection;
 	gboolean force_connection;
 	gboolean auto_update;
+	gboolean conn_wlan_only;
 
 	// used by the settings dialog
 	GtkWidget *settings_window;
